#include "..\script_component.hpp"

/*
 * Function: forge_client_org_fnc_disband
 * Author: IDSolutions
 *
 * [Description]
 * Disbands an organization if requested by the owner.
 * This permanently deletes the organization from the database and cannot be undone.
 * Only the organization owner can perform this action.
 *
 * Arguments:
 * None
 *
 * Return Value:
 * Success <BOOLEAN> - True if organization was disbanded successfully, false otherwise
 *
 * Example:
 * [] call forge_client_org_fnc_disband
 *
 * Public: Yes
 */

// Get the organization store interface
private _store = call FUNC(verifyOrgStore);

// Attempt to delete the organization
// This will validate ownership permissions and handle database deletion
private _result = _store call ["deleteOrg", []];

// Provide feedback based on success or failure
if (_result) then {
    ["Organization disbanded successfully", "success", 5, "right"] call forge_client_misc_fnc_notify;
} else {
    ["Failed to disband organization", "error", 5, "right"] call forge_client_misc_fnc_notify;
};

// Return the result for further operations
_result